@@ -6,7 +6,7 @@ namespace Tweeper;
  * @file
  * Tweeper - a Twitter to RSS web scraper.
  *
- * Copyright (C) 2013-2018  Antonio Ospite <ao2@ao2.it>
+ * Copyright (C) 2013-2020  Antonio Ospite <ao2@ao2.it>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,7 +36,7 @@ date_default_timezone_set('UTC');
  */
 class Tweeper {
 
-  private static $userAgent = "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:64.0) Gecko/20100101 Firefox/64.0";
+  private static $userAgent = "Mozilla/5.0";
   private static $maxConnectionTimeout = 5;
   private static $maxConnectionRetries = 5;
 
@@ -53,11 +53,14 @@ class Tweeper {
    *   Enables showing multimedia content (images, videos) directly in the
    *   item description (enabled by default). Only some stylesheets supports
    *   this functionality (twitter, instagram, dilbert).
+   * @param bool $verbose_output
+   *   Enables showing non-fatal errors like XML parsing errors.
    */
-  public function __construct($generate_enclosure = FALSE, $show_usernames = TRUE, $show_multimedia = TRUE) {
+  public function __construct($generate_enclosure = FALSE, $show_usernames = TRUE, $show_multimedia = TRUE, $verbose_output = TRUE) {
     $this->generate_enclosure = $generate_enclosure;
     $this->show_usernames = $show_usernames;
     $this->show_multimedia = $show_multimedia;
+    $this->verbose_output = $verbose_output;
   }
 
   /**
@@ -83,6 +86,46 @@ class Tweeper {
     return Tweeper::epochToRssDate($timestamp);
   }
 
+  /**
+   * Convert Twitter mobile date to the date format expected in a RSS document.
+   */
+  public static function twitterToRssDate($date) {
+    // Twitter uses relative timestamps in minutes for recent tweets.
+    if (preg_match('/^(\d+)m$/', $date, $matches)) {
+      $timestamp = strtotime("+" . $matches[1] . " min", time());
+      if (FALSE === $timestamp) {
+        $timestamp = 0;
+      }
+    }
+    else {
+      /*
+       * In case the time is specified put it after the date,
+       * to make it recognized by strptime().
+       */
+      if (preg_match('/(.*) - (.*)/', $date, $matches)) {
+        $date = $matches[2] . " " . $matches[1];
+      }
+
+      $timestamp = strtotime($date);
+      if (FALSE === $timestamp) {
+        $timestamp = 0;
+      }
+
+      /*
+       * The twitter mobile UI usually only specifies the month and the day, so
+       * strtotime($date) may interpret the date as being in the future.
+       *
+       * If the date is in the future it is probably in the same day but in the
+       * previous year.
+       */
+      if ($timestamp > time()) {
+        $timestamp = strtotime('-1 years', $timestamp);
+      }
+    }
+
+    return Tweeper::epochToRssDate($timestamp);
+  }
+
   /**
    * Convert string to UpperCamelCase.
    */
@@ -114,20 +157,18 @@ class Tweeper {
   /**
    * Get the contents from a URL.
    */
-  private static function getUrlContents($url) {
+  private static function getUrlContents($url, $user_agent = NULL) {
     $ch = curl_init($url);
-    curl_setopt_array($ch, array(
+    curl_setopt_array($ch, [
       CURLOPT_HEADER => FALSE,
       CURLOPT_CONNECTTIMEOUT => Tweeper::$maxConnectionTimeout,
       // Follow http redirects to get the real URL.
       CURLOPT_FOLLOWLOCATION => TRUE,
       CURLOPT_COOKIEFILE => "",
       CURLOPT_RETURNTRANSFER => TRUE,
-      CURLOPT_SSL_VERIFYHOST => FALSE,
-      CURLOPT_SSL_VERIFYPEER => FALSE,
-      CURLOPT_HTTPHEADER => array('Accept-language: en'),
-      CURLOPT_USERAGENT => Tweeper::$userAgent,
-    ));
+      CURLOPT_HTTPHEADER => ['Accept-language: en'],
+      CURLOPT_USERAGENT => isset($user_agent) ? $user_agent : Tweeper::$userAgent,
+    ]);
     $contents = Tweeper::curlExec($ch);
     curl_close($ch);
 
@@ -137,19 +178,17 @@ class Tweeper {
   /**
    * Get the headers from a URL.
    */
-  private static function getUrlInfo($url) {
+  private static function getUrlInfo($url, $user_agent = NULL) {
     $ch = curl_init($url);
-    curl_setopt_array($ch, array(
+    curl_setopt_array($ch, [
       CURLOPT_HEADER => TRUE,
       CURLOPT_NOBODY => TRUE,
       CURLOPT_CONNECTTIMEOUT => Tweeper::$maxConnectionTimeout,
       // Follow http redirects to get the real URL.
       CURLOPT_FOLLOWLOCATION => TRUE,
       CURLOPT_RETURNTRANSFER => TRUE,
-      CURLOPT_SSL_VERIFYHOST => FALSE,
-      CURLOPT_SSL_VERIFYPEER => FALSE,
-      CURLOPT_USERAGENT => Tweeper::$userAgent,
-    ));
+      CURLOPT_USERAGENT => isset($user_agent) ? $user_agent : Tweeper::$userAgent,
+    ]);
 
     $ret = Tweeper::curlExec($ch);
     if (FALSE === $ret) {
@@ -170,7 +209,7 @@ class Tweeper {
    * Generate an RSS <enclosure/> element.
    */
   public static function generateEnclosure($url) {
-    $supported_content_types = array(
+    $supported_content_types = [
       "application/octet-stream",
       "application/ogg",
       "application/pdf",
@@ -189,7 +228,7 @@ class Tweeper {
       "video/mp4",
       "video/mpeg",
       "video/ogg",
-    );
+    ];
 
     $url_info = Tweeper::getUrlInfo($url);
     if (FALSE === $url_info) {
@@ -271,13 +310,13 @@ class Tweeper {
 
     $encoder = new XmlEncoder();
     $normalizer = new ObjectNormalizer();
-    $serializer = new Serializer(array($normalizer), array($encoder));
+    $serializer = new Serializer([$normalizer], [$encoder]);
 
-    $serializer_options = array(
+    $serializer_options = [
       'xml_encoding' => "UTF-8",
       'xml_format_output' => TRUE,
       'xml_root_node_name' => $root_node_name,
-    );
+    ];
 
     $xml_data = $serializer->serialize($data, 'xml', $serializer_options);
     if (!$xml_data) {
@@ -336,15 +375,17 @@ class Tweeper {
     // If there is a host-specific method to get the XML data, use it!
     $get_xml_host_method = 'getXml' . Tweeper::toUpperCamelCase($host, '.');
     if (method_exists($this, $get_xml_host_method)) {
-      $xml_data = call_user_func_array(array($this, $get_xml_host_method), array($html));
+      $xml_data = call_user_func_array([$this, $get_xml_host_method], [$html]);
       $xmlDoc->loadXML($xml_data);
     }
     else {
       $xmlDoc->loadHTML($html);
     }
 
-    foreach (libxml_get_errors() as $xml_error) {
-      Tweeper::logXmlError($xml_error);
+    if ($this->verbose_output) {
+      foreach (libxml_get_errors() as $xml_error) {
+        Tweeper::logXmlError($xml_error);
+      }
     }
     libxml_clear_errors();
     libxml_use_internal_errors($xml_errors_value);
@@ -362,7 +403,7 @@ class Tweeper {
       return NULL;
     }
 
-    $stylesheet_contents = Tweeper::getUrlContents($stylesheet);
+    $stylesheet_contents = file_get_contents($stylesheet);
     if (FALSE === $stylesheet_contents) {
       trigger_error("Cannot open $stylesheet", E_USER_WARNING);
       return NULL;
@@ -393,7 +434,7 @@ class Tweeper {
 
     if (TRUE === $validate_scheme) {
       $scheme = $url["scheme"];
-      if (!in_array($scheme, array("http", "https"))) {
+      if (!in_array($scheme, ["http", "https"])) {
         trigger_error("unsupported scheme: $scheme", E_USER_WARNING);
         return NULL;
       }
@@ -414,7 +455,15 @@ class Tweeper {
       return NULL;
     }
 
-    $html = Tweeper::getUrlContents($src_url);
+    // Override User-Agent for twitter.com to force it to serve the mobile UI.
+    if ($host == "twitter.com") {
+      $user_agent = "Mozilla/5.0 (Linux; U; Android 4.3; en-us; SM-N900T Build/JSS15J)";
+    }
+    else {
+      $user_agent = NULL;
+    }
+
+    $html = Tweeper::getUrlContents($src_url, $user_agent);
     if (FALSE === $html) {
       trigger_error("Failed to retrieve $src_url", E_USER_WARNING);
       return NULL;
@@ -422,7 +471,7 @@ class Tweeper {
 
     $preprocess_html_host_method = 'preprocessHtml' . Tweeper::toUpperCamelCase($host, '.');
     if (method_exists($this, $preprocess_html_host_method)) {
-      $html = call_user_func_array(array($this, $preprocess_html_host_method), array($html));
+      $html = call_user_func_array([$this, $preprocess_html_host_method], [$html]);
     }
 
     $xmlDoc = $this->htmlToXml($html, $host);
